@@ -821,12 +821,12 @@ def least_squares(
 
     ftol, xtol, gtol = check_tolerance(ftol, xtol, gtol, method)
 
+    if method == 'trf':
+        x0 = make_strictly_feasible(x0, lb, ub, rstep=0)
+
     def fun_wrapped(x):
         return np.atleast_1d(fun(x, *args, **kwargs))
 
-    if method == 'trf':
-        x0 = make_strictly_feasible(x0, lb, ub)
-
     f0 = fun_wrapped(x0)
 
     if f0.ndim != 1:
